@@ -1,18 +1,20 @@
 import functools
 import inspect
 import json
-from typing import Callable, AnyStr
+from typing import Callable, Union, Dict
 import warnings
 
-from arcane.core import CLIENTS_PARAM, ALL_CLIENTS_RIGHTS, UserRightsEnum, RightsLevelEnum
+import backoff
+from firebase_admin import auth as firebase_auth
+from flask import request
+from google.cloud.exceptions import ServiceUnavailable
+
+from arcane.core import CLIENTS_PARAM, ALL_CLIENTS_RIGHTS, RightsLevelEnum, GOOGLE_EXCEPTIONS_TO_RETRY
 from arcane.datastore import Client as DatastoreClient
 from arcane.pubsub import Client as PubSubClient
-from flask import request
-from firebase_admin import auth as firebase_auth
+
 
 from .tracking import send_tracking_information
-from .log import adscale_log
-from .services import ServiceEnum
 
 CLIENTS_KIND = 'base-clients'
 
@@ -28,7 +30,7 @@ class IncorrectAccessRightsRouteImplementation(TypeError):
 
 def check_access_rights(service: str,
                         required_rights: int,
-                        service_user_right: UserRightsEnum,
+                        service_user_right: str,  # should be something like ValueOf[UserRightsEnum]
                         datastore_client: DatastoreClient,
                         pubsub_client: PubSubClient,
                         receive_rights_per_client: bool=False,
@@ -43,9 +45,9 @@ def check_access_rights(service: str,
     as the last positional argument or as an named argument.
     """
     if required_rights == RightsLevelEnum.NONE and not receive_rights_per_client:
-        def check_rights(job_func: Callable) -> Callable:
+        def dummy_check_rights(job_func: Callable) -> Callable:
             return job_func
-        return check_rights
+        return dummy_check_rights
 
     def check_rights(job_func: Callable) -> Callable:
 
@@ -77,7 +79,7 @@ def check_access_rights(service: str,
                 token = str(request.headers.get('Authorization', '')).split(' ').pop()
                 origin = str(request.headers.get('origin', ''))
                 try:
-                    claims = firebase_auth.verify_id_token(token)
+                    claims = verify_token(token)
                 except (firebase_auth.ExpiredIdTokenError,
                         firebase_auth.RevokedIdTokenError,
                         firebase_auth.InvalidIdTokenError) as e:
@@ -89,8 +91,9 @@ def check_access_rights(service: str,
                 try:
                     send_tracking_information(email=claims['email'], service=service, function_name=job_func.__name__, origin=origin,
                                             project=project, pubsub_client=pubsub_client, datastore_client=datastore_client)
-                except KeyError:
-                    adscale_log(ServiceEnum.TRACKING, f'The user email is not contained in the claims')
+                except (KeyError, ServiceUnavailable):
+                    # Failing to send tracking info should not impact feature usability
+                    pass
 
                 rights = claims.get('features_rights', {}).get(service_user_right, RightsLevelEnum.NONE)
                 authorized_clients = claims.get(CLIENTS_PARAM, [])
@@ -127,7 +130,13 @@ def check_access_rights(service: str,
     return check_rights
 
 
-def get_user_email(token: AnyStr=None, auth_enabled=True) -> str:
+@backoff.on_exception(backoff.expo, (GOOGLE_EXCEPTIONS_TO_RETRY), max_tries=3)
+def verify_token(token: str) -> Dict:
+    claims = firebase_auth.verify_id_token(token)
+    return claims
+
+
+def get_user_email(token: str=None, auth_enabled=True) -> Union[str, None]:
     """
     retrieves user id from connexion header.
     may raise ValueError if Authorization token is invalid or if token did not have uid specified
@@ -148,7 +157,7 @@ def get_user_email(token: AnyStr=None, auth_enabled=True) -> str:
         raise ValueError(f'Uid key is mapped to {repr(mail_str)} (type: {type(mail_str)}) value in claims')
     return mail_str
 
-def get_user_uid(token: AnyStr=None) ->str:
+def get_user_uid(token: str=None) ->str:
     """
     Retrieve user firebase uid from connexion header.
 
